<?php

namespace Drupal\editoria11y\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Xss;

/**
 * Class Editoria11ySettings.
 */
class Editoria11ySettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'editoria11y_form_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'editoria11y.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('editoria11y.settings');
    $form['setup'] = array(
      '#type' => 'fieldset',
      '#title' => t('Inclusions &amp; exclusions'),
    );
    $form['setup']['content_root'] = [
      '#title' => $this->t("Check content in this container"),
      '#type' => 'textfield',
      '#placeholder' => 'body',
      '#description' => $this->t('If all editorial content is in <strong>one</strong> element (e.g., <code>main</code> or <code>#content</code>), provide that single <a href="https://developer.mozilla.org/en-US/docs/Learn/CSS/Building_blocks/Selectors">selector</a>.'),
      '#default_value' => $config->get('content_root'),
    ];
    $form['setup']['ignore_containers'] = [
      '#title' => $this->t("Skip over these elements"),
      '#type' => 'textfield',
      '#placeholder' => '#toolbar-administration, .contextual-region > nav, .search-results',
      '#description' => $this->t('Provide a comma-separated list of selectors for elements to ignore, such as <code>.my-embedded-social-media-feed, #sidebar-menu</code>.'),
      '#default_value' => $config->get('ignore_containers'),
    ];
    $form['setup']['no_load'] = [
      '#title' => $this->t("Disable the scanner if these elements are detected"),
      '#type' => 'textfield',
      '#placeholder' => '#quickedit-entity-toolbar, #layout-builder',
      '#description' => $this->t('Provided a comma-separated list of selectors unique to pages or states that should not be scanned; e.g, during inline editing  (<code>#inline-editor-open</code>) or on pages without user-editable content (<code>.node-261, .front</code>).'),
      '#default_value' => $config->get('no_load'),
    ];
    $form['results'] = array(
      '#type' => 'fieldset',
      '#title' => t('Test results'),
    );
    $form['results']['assertiveness'] = [
      '#title' => $this->t("Open the issue details panel automatically when new issues are detected"),
      '#type' => 'radios',
      '#options' => array(
        'smart' => $this->t('When nodes are created or changed'),
        'assertive' => $this->t('Always'),
        'polite' => $this->t('Never')
      ),
      '#description' => $this->t('"Always" is not recommended for sites with multiple editors.'),
      '#default_value' => $config->get('assertiveness'),
    ];
    $form['results']['allow_overflow'] = [
      '#title' => $this->t("Force these containers to allow overflow when tips are open"),
      '#type' => 'textfield',
      '#description' => $this->t('Page elements themed with overflow:hidden will truncate tooltips that appear inside them. Overflow:visible will be applied to elements in this comma-separated list of selectors when the editor tries to view an alert explanation tooltip.'),
      '#default_value' => $config->get('allow_overflow'),
    ];
    $form['results']['embedded_content_warning'] = [
      '#title' => $this->t("Warn that these elements need manual review"),
      '#type' => 'textfield',
      '#description' => $this->t('Provide a comma-separated list of selectors for elements with potentially complex issues, e.g.: <code>.my-embedded-feed, #my-social-link-block</code>.<br> Note that this test currently only runs during a "full check," and it <strong>ignores the ignore list</strong>: you are specifying that this element has not been checked.'),
      '#default_value' => $config->get('embedded_content_warning'),
    ];
    $form['text'] = array(
      '#markup' => '<p>Configuration and troubleshooting tips: <ol><li><a href="/admin/people/permissions">Editorial roles</a> need to have the "View Editoria11y" permission.</li><li>If permissions are correct but the toggle still does not appear, selectors in the "disable the scanner if..." field could be present on the page.</li><li>If the toggle appears but errors are never found, make sure the "check content in" is set to the element that contains the content, and "skip over these" <strong><em>is not</em></strong>.</li><li>Note that an invalid configuration will prevent scanning and throw a JavaScript error; <a href="https://developer.mozilla.org/en-US/docs/Tools/Browser_Console">check the browser console</a></li></ol>',
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('editoria11y.settings')
      ->set('ignore_containers', Xss::filter($form_state->getValue('ignore_containers')))
      ->set('assertiveness', $form_state->getValue('assertiveness'))
      ->set('no_load', Xss::filter($form_state->getValue('no_load')))
      ->set('content_root', Xss::filter($form_state->getValue('content_root')))
      ->set('allow_overflow', Xss::filter($form_state->getValue('allow_overflow')))
      ->set('embedded_content_warning', Xss::filter($form_state->getValue('embedded_content_warning')))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
